CURVE_DAO_OWNERSHIP = {
    "agent": "0x40907540d8a6C65c637785e8f8B742ae6b0b9968",
    "voting": "0xE478de485ad2fe566d49342Cbd03E49ed7DB3356",
    "token": "0x5f3b5DfEb7B28CDbD7FAba78963EE202a494e2A2",
    "quorum": 30,
}

CURVE_DAO_PARAM = {
    "agent": "0x4eeb3ba4f221ca16ed4a0cc7254e2e32df948c5f",
    "voting": "0xbcff8b0b9419b9a88c44546519b1e909cf330399",
    "token": "0x5f3b5DfEb7B28CDbD7FAba78963EE202a494e2A2",
    "quorum": 15,
}

EMERGENCY_DAO = {
    "forwarder": "0xf409Ce40B5bb1e4Ef8e97b1979629859c6d5481f",
    "agent": "0x00669DF67E4827FCc0E48A1838a8d5AB79281909",
    "voting": "0x1115c9b3168563354137cdc60efb66552dd50678",
    "token": "0x4c0947B16FB1f755A2D32EC21A0c4181f711C500",
    "quorum": 51,
}

GAUGE_CONTROLLER = "0x2F50D538606Fa9EDD2B11E2446BEb18C9D5846bB"
VOTING_ESCROW = "0x5f3b5DfEb7B28CDbD7FAba78963EE202a494e2A2"
veCRV = VOTING_ESCROW
CRV = "0xD533a949740bb3306d119CC777fa900bA034cd52"
CONVEX_VOTERPROXY = "0x989AEB4D175E16225E39E87D0D97A3360524AD80"
ADDRESSPROVIDER = "0x5ffe7FB82894076ECB99A30D6A32e969e6e35E98"  # On all chains


def get_dao_voting_contract(vote_type: str):
    target = get_target(vote_type)
    return target["voting"]


def get_target(vote_type: str):
    match vote_type:
        case "ownership":
            return CURVE_DAO_OWNERSHIP
        case "parameter":
            return CURVE_DAO_PARAM
        case "emergency":
            return EMERGENCY_DAO



import sys
import warnings
from typing import Dict, List, Tuple

import boa
from rich.console import Console as RichConsole

warnings.filterwarnings("ignore")

logger = RichConsole(file=sys.stdout)


class MissingVote(Exception):
    """Exception raised when a vote ID is invalid."""


def prepare_evm_script(target: Dict, actions: List[Tuple], etherscan_api_key: str):
    """Generates EVM script to be executed by AragonDAO contracts.

    Args:
        target (dict): CURVE_DAO_OWNERSHIP / CURVE_DAO_PARAMS / EMERGENCY_DAO
        actions (list(tuple)): ("target addr", "fn_name", *args)

    Returns:
        str: Generated EVM script.
    """
    aragon_agent = boa.from_etherscan(
        target["agent"], name="AragonAgent", api_key=etherscan_api_key
    )
    evm_script = bytes.fromhex("00000001")

    for action in actions:
        address, fn_name, *args = action
        contract = boa.from_etherscan(
            address=address, name="TargetContract", api_key=etherscan_api_key
        )
        contract_function = getattr(contract, fn_name)

        calldata = contract_function.prepare_calldata(*args)
        agent_calldata = aragon_agent.execute.prepare_calldata(address, 0, calldata)

        length = bytes.fromhex(hex(len(agent_calldata.hex()) // 2)[2:].zfill(8))
        evm_script = (
            evm_script
            + bytes.fromhex(aragon_agent.address[2:])
            + length
            + agent_calldata
        )

    return evm_script



from typing import Dict

import boa
from hexbytes import HexBytes

from .ipfs import pin_to_ipfs  # noqa: F401
from .simulate import simulate  # noqa: F401
from .vote_utils import prepare_evm_script  # noqa: F401


def create_vote(
    target: Dict, evm_script: str, ipfs_hash_of_description: str, etherscan_api_key: str, 
):
    voting = boa.from_etherscan(
        target["voting"], "AragonVoting", api_key=etherscan_api_key
    )
    assert voting.canCreateNewVote(boa.env.eoa)
    return voting.newVote(
        HexBytes(evm_script), f"ipfs:{ipfs_hash_of_description}", False, False
    )



import pprint
import sys

import boa
from rich.console import Console as RichConsole

from .addresses import CONVEX_VOTERPROXY

logger = RichConsole(file=sys.stdout)


def simulate(vote_id: int, voting_contract: str, etherscan_api_key: str):
    logger.info("--------- SIMULATE VOTE ---------")

    voting_contract = boa.from_etherscan(
        voting_contract, "AragonVoting", api_key=etherscan_api_key
    )

    # print vote details to console first:
    logger.info("Vote stats before Convex Vote:")
    vote_stats = voting_contract.getVote(vote_id)
    logger.info(pprint.pformat(vote_stats, indent=4))

    # vote
    logger.info("Simulate Convex 'yes' vote")
    with boa.env.prank(CONVEX_VOTERPROXY):
        voting_contract.vote(vote_id, True, False)

    # sleep for a week so it has time to pass
    num_seconds = voting_contract.voteTime()
    boa.env.time_travel(seconds=num_seconds)

    # get vote stats:
    logger.info("Vote stats after 1 week:")
    vote_stats = voting_contract.getVote(vote_id)
    pprint.pformat(vote_stats, indent=4)

    # moment of truth - execute the vote!
    logger.info("Simulate proposal execution")
    with boa.env.prank(boa.env.generate_address()):
        voting_contract.executeVote(vote_id)
    logger.info("Vote Executed!")



import requests


def pin_to_ipfs(description: str, pinata_token: str):
    """Uploads vote description to IPFS via Pinata and returns the IPFS hash.

    NOTE: Needs environment variables for Pinata IPFS access. Please
    set up an IPFS project to generate API key and API secret!
    """

    url = "https://api.pinata.cloud/pinning/pinJSONToIPFS"
    headers = {
        "Authorization": f"Bearer {pinata_token}",
        "Content-Type": "application/json",
    }
    payload = {
        "pinataContent": {"text": description},
        "pinataMetadata": {"name": "pinnie.json"},
        "pinataOptions": {"cidVersion": 1},
    }
    response = requests.request("POST", url, json=payload, headers=headers)

    assert (
        200 <= response.status_code < 400
    ), f"POST to IPFS failed: {response.status_code}"

    return response.json()["IpfsHash"]
